% SPLIBAS Computes polynomial spline basis.
% USAGE
%   [B,x,k,breaks]=splibas(breaks,evennum,k,x,order);
% INPUTS
%   breaks  : user specified breakpoint sequence
%             (default: evenly spaced non-repeated breakpoints)
%   evennum : non-zero if breakpoints are all even
%   k       : polynomial order of the spline's pieces (default: 3, cubic)
%   x       : vector of the evaluation points (default: k-point averages of breakpoints)
%   order   : the order of differentiation (default: 0)
%             if a vector, SPLIBAS returns a cell array 
%             otherwise it returns a matrix
% OUTPUTS
%   B : a kxn basis matrix
%   x : evaluation points (useful if defaults values are computed) 
%
% Note: The number of basis functions is n=length(breaks)+k-1
%
% USES: splidop
%
% See also: SPLINODE, SPLIDEF, SPLIDOP, FUNBAS

% Copyright (c) 1997-2000, Paul L. Fackler & Mario J. Miranda
% paul_fackler@ncsu.edu, miranda.4@osu.ed

function [B0,B1,B2] = SM_splibas(breaks,x_grid,D1,D2,stop)
% This function outputs the basis matrices of the approximant and the
% first (B1) and second (B2) derivatives of a CUBIC spline ONLY for a
% SINGLE point x
% This requires the inputs of the 'derivative operator matrices' {D1,D2}
% which have to be pre-computed. The code to compute these is simply:
% [D1,D2] = SM_splidop(xgrid0);
% Note that SM_splidop *also* only is designed for the cubic spline
%%%% STOP = The number of matrices created. Note that when computing the
%%%% FOC and the Jacobian then stop=2 (compute 2nd and 1st order), if just
%%%% evaluating the function then stop==3
%__________________________________________________________________________
n_breaks                            = length(breaks);
n_x                                 = size(x_grid,1);
%__________________________________________________________________________
% Augment the breakpoint sequence 
a                                   = breaks(1); 
b                                   = breaks(n_breaks);
augbreaks(1:3)                      = a;
augbreaks(4:n_breaks+3)             = breaks;
augbreaks(n_breaks+4:n_breaks+6)    = b;
%__________________________________________________________________________
% Recursively determine the values of a k-order basis matrix.
% This is placed in an (m x k+1-order) matrix
bas         = zeros(n_x,4);
bas(:,1)    = 1.0;
X2          = zeros(n_x,n_breaks);
X1          = zeros(n_x,n_breaks+1);
B0          = zeros(n_x,n_breaks+2);
%__________________________________________________________________________
for x_idx = 1:n_x
    %______________________________________________________________________
    % 1. Find break points and set ind
    x       = x_grid(x_idx);
    ind     = find(x > breaks, 1, 'last');
    if (isempty(ind))
        ind = 1;
    end
    ind     = ind + 3;
    %______________________________________________________________________
    % 2. Update basis matrix
    for j=1:stop
        for negjj = 0:(j-1)
            jj              = j - negjj;
            b0              = augbreaks(ind+jj-j);
            b1              = augbreaks(ind+jj);
            temp            = bas(x_idx,jj) / (b1-b0);
            bas(x_idx,jj+1) = (x - b0).*temp + bas(x_idx,jj+1);
            bas(x_idx,jj)   = (b1 - x).*temp;
        end
        %__________________________________________________________________
        % 3. Put values into basis matrices
        if j==1
            % 2nd order terms
            X2(x_idx,ind-3:ind-2)   = bas(x_idx,1:2);
        elseif j==2
            % 1st order terms
            X1(x_idx,ind-3:ind-1)   = bas(x_idx,1:3);
        elseif j==3
            % 0th order terms
            B0(x_idx,ind-3:ind)     = bas(x_idx,1:4);
        end
    end
end
%__________________________________________________________________________
% Apply derivative operator
B1  = X1*D1;
B2  = X2*D2;
%__________________________________________________________________________
end
  
  
  