% SPLIDOP Creates differential operator matrices for polynomial splines.
% USEAGE
%   d=splidop(breaks,evenflag,k,order);  
%
% Computes the matrix operator that maps a vector of spline
% coefficients into the vector of coefficients of the derivative.
% Example:
%    g(x)=B^k(x)*c;
%    g'(x)=B^{k-1}(x)*splidop(n,a,b,1)*c;
%    g"(x)=B^{k-1}(x)*splidop(n,a,b,2)*c;
% Integrals are computed with ORDER<1 (anti-derivatives).
%    int_a^x g(x)=B^{k+1}(x)*splidop(n,a,b,-1)
%
% D is returned as a cell array of sparse matrices w/ abs(order) 
%   elements; to view it use full(D{i})
%
% Most users will not need this function; 
%   use SPLIBAS, FUNEVAL or FUNDOP instead.
%
% See also: SPLIBAS, FUNEVAL, FUNDOP.

% Copyright (c) 1997-2000, Paul L. Fackler & Mario J. Miranda
% paul_fackler@ncsu.edu, miranda.4@osu.edu

function [D1,D2] =SM_splidop(breaks)
% This function outputs the D matrices for spline derivatives for a CUBIC
% SPLINE only.
n_breaks                            = length(breaks);
n                                   = n_breaks + 2;
%__________________________________________________________________________
% Make augbreaks
a                                   = breaks(1);
b                                   = breaks(n_breaks);
augbreaks                           = zeros(n_breaks+4,1);
augbreaks(1:2)                      = a;
augbreaks(3:n_breaks+2)             = breaks;
augbreaks(n_breaks+3:n_breaks+4)    = b;
%__________________________________________________________________________
% First derivative operator
temp            = 3./(augbreaks(4:n+2)-augbreaks(1:n-1));
D1              = zeros(n-1,n);
for i = 1:n-1;
    D1(i,i)     = -temp(i);
    D1(i,i+1)   =  temp(i);
end
%__________________________________________________________________________
% Second derivative operator
temp            = 2./(augbreaks(4:n+1)-augbreaks(2:n-1));
D2              = zeros(n-2,n-1);
for i = 1:n-2
    D2(i,i)     = -temp(i);
    D2(i,i+1)   =  temp(i);
end
D2            = D2*D1;
%__________________________________________________________________________
% D1          = spdiags([-temp temp],0:1,n-1,n);            % MATLAB COMMAND
% D2          = spdiags([-temp temp],0:1,n-2,n-1)*D1;       % MATLAB COMMAND
end












